
# q-expansion of newform 1050.2.a.e, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1050
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [1], [2], [-1], [1], [4], [-7], [1], [3], [6], [-3], [1], [12], [-11], [-3], [5], [12], [4], [14], [-2], [3], [10], [10], [0], [17], [18], [-4], [-6], [-14], [8], [-4], [-4], [5], [22], [-18], [19], [2], [12], [10], [-18], [-13], [-2], [-27], [-24], [15], [13], [-7], [14], [-20], [-24], [-26], [-15], [5], [-11], [18], [-20], [2], [0], [20], [0], [22], [34], [-18], [17], [-3], [-27], [-26], [-1], [-10], [-9], [-13], [4], [-5], [20], [22], [-3], [16], [-14], [-25], [34], [27], [-34], [7], [12], [-16], [17], [32], [-36], [39], [8], [-22], [30], [-27], [2], [0], [39], [8], [-28], [15], [-18], [-41], [14], [19], [-34], [27], [18], [-45], [28], [-28], [-18], [-14], [38], [26], [-26], [-2], [-6], [-14], [-6], [-22], [-27], [20], [6], [-42], [-39], [4], [-34], [1], [-19], [11], [3], [-20], [2], [34], [40], [20], [-38], [-42], [20], [-14], [-38], [18], [50], [3], [-40], [-43], [46], [40], [-32], [-32], [5], [-29], [-50], [11], [-19], [-20], [25], [14], [0], [32], [40], [44], [-44], [-42], [-24], [4], [2], [-41], [0], [-26], [19], [-5], [17], [-4], [-63], [28], [-45], [9], [-19], [-42], [-39], [50], [-37], [0], [18], [64], [54], [-31], [0], [16], [28], [-24], [30], [8], [27], [5], [-2], [14], [0], [-18], [37], [-47], [10], [9], [4], [-17], [-42], [58], [-10], [34], [-6], [-34], [35], [2], [64], [32], [6], [7], [45], [32], [15], [54], [8], [-68], [6], [-30], [24], [-28], [-22], [8], [-8], [64], [70], [-12], [4], [-18], [66], [70], [50], [-24], [22], [-21], [-70], [-75], [60], [-15], [-20], [68], [-32], [44], [17], [7], [-42], [44], [-14], [58], [50], [-74], [-14], [-52], [72], [-13], [-2], [-51], [-62], [1], [45], [13], [-39], [55], [23], [64], [-4], [-28], [-76], [-40], [-10], [55], [-47], [-76], [44], [-48], [21], [4], [-5], [37], [-70], [-19], [50], [-50], [44], [60], [-32], [66], [-36], [21], [2], [-59], [-50], [20], [53], [-54], [4], [-12], [-13], [42], [-33], [-3], [-30], [-66], [-59], [30], [-8], [37], [-10], [-12], [39], [40], [0], [-86], [-46], [50], [4], [46], [-48], [31], [62], [-48], [-55], [46], [17], [-19], [21], [-37], [-68], [-20], [-63], [-26], [-11], [-66], [2], [-44], [-8], [27], [-8], [-42], [3], [-60], [58], [24], [-19], [-93], [19], [90], [-25], [-30], [51], [-6], [-59], [-92], [78], [-72], [60], [70], [15], [-26], [-44], [9], [36], [-44], [57], [51], [99], [-16], [-15], [24], [-72], [55], [28], [2], [28], [-20], [-20], [76], [-24], [-54], [-82], [-20], [62], [-17], [-24], [-6], [-50], [-5], [46], [49], [39], [-20], [18], [-19], [-54], [-92], [91], [4], [-27], [2], [58], [-14], [32], [20], [54], [12], [-49], [-43], [-56], [-5], [71], [5], [-18], [-25], [91], [27], [7], [-100], [16], [74], [-30], [100], [14]]
